/** Update component component properties */
try{
    // Init component \see data in component html
    function onInitComponent(data){
        //console.log("[onInitComponent][" + data.source() + "][" + data + "]");
        return updateMainFrame(data);
    }

    // compones-mainframe.html callback
    function onGuiChangeGateway(sender,gateway,data) {
        //console.log("[onGuiChangeGateway][" + sender.source + "][" + gateway.getProtocol() + "][" + data + "]");
        return updateProtocol(gateway.getProtocol(),data);
    }

    /** On Update Component Function (Required)
     * Update component when some change has occured
     */
    function onUpdateComponent(data){
        //console.log("[onUpdateComponent][" + data.source() + "][" + data + "]");
        return updateProtocol(data.protocol(),data);
    };

    function updateProtocol(strProtocol,data){
        switch(strProtocol){
            case "knx":{
                updateKnx(data);
            }break;
            case "vimar":{
                updateVimar(data);
            }break;
            case "hue":{
                updateHue(data);
            }break;
            case "ghost":{
                updateGhost(data);
            }break;
            default:{
                return updateDefault(data);
            }
        }
        return true;
    }

    function updateMainFrame(data){
        var $Elem = undefined;
        $("[eve-manager-id='UID']").text(data.node().UId);
        $("[eve-manager-id='type']").text(data.node().type);

        //Set name
        $Elem = data.$properties().find("[eve-manager-id='name']");
        $Elem.val(data.node().getParamVal("name"));

        //Set html name
        $Elem = $("#"+ data.node().id +" .components-text");
        $Elem.text(data.node().getParamVal("name"));

        //Build gateways
        $Elem = data.$properties().find("[eve-manager-id='gateway']");
        $.publish("/view/gateways/refresh/dependencies");

        //Set gateways
        var strProtocol;
        $Elem = data.$properties().find("[eve-manager-id='gateway'] option");
        data.$properties().find(".protocols").hide();												//Hide all protocols
        for(var i = 0; i < $Elem.length; ++i){
            var $currElem = $($Elem[i]);
            var oNode = $currElem.data("eveManagerNode");
            //Find gateway
            if(oNode === data.node().getParamVal("gateway")){
                $currElem.prop("selected",true);													//Select gateway
                if(oNode == undefined){ break; }													//If none gateway is selected break for cicle

                var oGateway = oNode.getData();														//Get node data (gateway)

                strProtocol = oGateway.getProtocol();												//Get protocol id
                data.$properties().find("[eve-manager-protocol='"+ strProtocol +"']").show();		//Show protocol view
                break;
            }
        }

        //Set enable history
        $Elem = data.$properties().find("[eve-manager-id='history']");
        $Elem.prop("checked",data.node().getParamVal("history"));

        //Set enable set last value
        $Elem = data.$properties().find("[eve-manager-id='savelastvalue']");
        $Elem.prop("checked",data.node().getParamVal("savelastvalue"));

        //Set enable cache
        $Elem = data.$properties().find("[eve-manager-id='cache']");
        $Elem.prop("checked", data.node().getParamVal("cache"));

        //Set tags
        $Elem = $componentProperties.find("[eve-manager-id='tags']");
        var resTags = data.node().getParamVal("tags");
        var $selectedTags = "[";

        if (resTags != "") {
            var res = resTags.split("|");
            for (var k = 0; k < res.length; k++) {
                if ($selectedTags.length > 1) $selectedTags += ",";
                $selectedTags += "{\"id\": \"" + res[k] + "\", \"text\": \"" + res[k] + "\"}";
            }
        }
        $selectedTags += "]";
        $Elem.select2("data", jQuery.parseJSON($selectedTags));

        //Set icon
        $Elem = data.$properties().find("[eve-manager-id='icons']");
        var aIcons = data.node().getParamVal("icons");
        $.publish("/view/components/" + data.node().id + "/refresh/parameter", [data.node(), "icons"]);

        //Set customData
        $Elem = $componentProperties.find("[eve-manager-id='customData']");
        $Elem.prop("value", data.node().getParamVal("customData"));
        $Elem.empty();
        return true;
    }

    function updateKnx(data){
        var $Elem = undefined;
        //Set KNX (RED) write address
        $Elem = data.$properties().find("[eve-manager-id='knx/red/write-address']");
        var aValue = data.node().getParamVal("knx/red/write-address").split("/");
        for (var i = 0; i < aValue.length; ++i) {
            $Elem[i].value = aValue[i];
        }

        //Set KNX (RED) feedback address
        $Elem = data.$properties().find("[eve-manager-id='knx/red/feedback-address']");
        aValue = data.node().getParamVal("knx/red/feedback-address").split("/");
        for (var i = 0; i < aValue.length; ++i) {
            $Elem[i].value = aValue[i];
        }

        //Set KNX (GREEN) write address
        $Elem = data.$properties().find("[eve-manager-id='knx/green/write-address']");
        var aValue = data.node().getParamVal("knx/green/write-address").split("/");
        for (var i = 0; i < aValue.length; ++i) {
            $Elem[i].value = aValue[i];
        }

        //Set KNX (GREEN) feedback address
        $Elem = data.$properties().find("[eve-manager-id='knx/green/feedback-address']");
        aValue = data.node().getParamVal("knx/green/feedback-address").split("/");
        for (var i = 0; i < aValue.length; ++i) {
            $Elem[i].value = aValue[i];
        }

        //Set KNX (BLUE) write address
        $Elem = data.$properties().find("[eve-manager-id='knx/blue/write-address']");
        var aValue = data.node().getParamVal("knx/blue/write-address").split("/");
        for (var i = 0; i < aValue.length; ++i) {
            $Elem[i].value = aValue[i];
        }

        //Set KNX (BLUE) feedback address
        $Elem = data.$properties().find("[eve-manager-id='knx/blue/feedback-address']");
        aValue = data.node().getParamVal("knx/blue/feedback-address").split("/");
        for (var i = 0; i < aValue.length; ++i) {
            $Elem[i].value = aValue[i];
        }

        $Elem = data.$properties().find("[eve-manager-id='knx/virtual-feedback']");
        $Elem.prop("checked", data.node().getParamVal("knx/virtual-feedback"));

        $Elem = data.$properties().find("[eve-manager-id='knx/read-init']");
        $Elem.prop("checked", data.node().getParamVal("knx/read-init"));
    }

    function updateVimar(data) {
        var json = blr.declarations.vimar.devices;
        $dropDown = data.$properties().find("[eve-manager-id='vimar/device-pid']");
        var $none = $("<option value='none' dictionary='None'>"+ _D("None") +"</option>");
        $dropDown.empty();
        $dropDown.append($none);

        //Parsing Vimar By Me Devices
        for(var i=0; i<json.length; ++i){
            var $option = $("<option></option>");

            //Check if is for Combo
            if(json[i].componentsType.indexOf("rgb-light") != -1){
                $option.text(json[i].itemCodes.toString() +" - "+ _D(json[i].label));
                $option.attr("title", _D(json[i].label));
                $option.val(json[i].pid);
                $option.data("VimarByMe",json[i]);

                //Append
                $dropDown.append($option);
            }
        }

        //Select currently FID
        var $selected = $dropDown.find("[value='"+ data.node().getParamVal("vimar/device-pid") +"']");
        var oVimarDevice = $selected.data("VimarByMe");

        $selected.prop("selected",true);

        var $fid = data.$properties().find("[eve-manager-id='vimar/device-fid']");
        var $none = $("<option value='none' dictionary='None'>"+ _D("None") +"</option>");

        //Empty combo selection
        $fid.empty();
        $fid.append($none);

        //Check if none device is selected
        if(oVimarDevice == undefined){
            $none.prop("selected",true);
            $fid.prop("disabled",true);
        }
        else{
            //Fill option parameters
            for(var i=0; i<oVimarDevice.fid.length; ++i){
                oVimarDevice.fid[i];
                var $option = $("<option></option>");

                $option.text(_D(oVimarDevice.fid[i].label));
                $option.val(oVimarDevice.fid[i].value);

                $fid.append($option);
            }

            $fid.removeProp("disabled");
        }

        //Select currently
        $fid.find("[value='"+ data.node().getParamVal("vimar/device-fid") +"']").prop("selected",true);

        //Set Vimar red physical address
        $Elem = data.$properties().find("[eve-manager-id='vimar/red/physical-address']");
        $Elem.val(data.node().getParamVal("vimar/red/physical-address"));

        //Set Vimar red group address
        $Elem = data.$properties().find("[eve-manager-id='vimar/red/group-address']");
        $Elem.val(data.node().getParamVal("vimar/red/group-address"));

        //Set Vimar green physical address
        $Elem = data.$properties().find("[eve-manager-id='vimar/green/physical-address']");
        $Elem.val(data.node().getParamVal("vimar/green/physical-address"));

        //Set Vimar green group address
        $Elem = data.$properties().find("[eve-manager-id='vimar/green/group-address']");
        $Elem.val(data.node().getParamVal("vimar/green/group-address"));

        //Set Vimar blue physical address
        $Elem = data.$properties().find("[eve-manager-id='vimar/blue/physical-address']");
        $Elem.val(data.node().getParamVal("vimar/blue/physical-address"));

        //Set Vimar blue group address
        $Elem = data.$properties().find("[eve-manager-id='vimar/blue/group-address']");
        $Elem.val(data.node().getParamVal("vimar/blue/group-address"));
    }

    function updateHue(data){
        var $Elem = data.$properties().find("[eve-manager-id='hue/id']");
        $Elem.val(data.node().getParamVal("hue/id"));
    }

    function updateGhost(data){
        //Set virtual feedback
        var $Elem = data.$properties().find("[eve-manager-id='ghost/virtual-feedback']");
        $Elem.prop("checked", data.node().getParamVal("ghost/virtual-feedback") );

        $Elem = data.$properties().find("[eve-manager-id='ghost/userdata']");
        $Elem.val(data.node().getParamVal("ghost/userdata"));
        updateCustomData(data.node());
    }

    function updateDefault(data){

    }
}
catch(err){
    console.error(err);
}
