/** Eve Manager user interface map class
 * 
 * @class Map
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 */
"use strict";

function Map(){
	var self = this;
	
	/** Map component id generator
     * @public
     * @type EveIdAssign
     */
    var m_oMapComponentIdAssign = new EveIdAssign();

    var init = function () {
        self.icon.position.x = -6*100;
        self.icon.position.y = -14*102;
    };


	
	/** Map background image path string
	 * @public
	 * @type string
	 * @default undefined
	 */
	this.background = "";
	
	/** Map background image position
     * @public
     * @type object
     * @default object
     */
    this.backgroundPosition = {
        x: 0,
        y: 0
    };
    
    /** Map background image size
     * @public
     * @type object
     * @default object
     */
    this.backgroundSize = {
        h: 0,
        w: 0
    };
	
	/** Map background visibility
	 * @public 
	 * @type boolean
	 * @default true
	 */
	this.imageVisible = true;
	
	/** Map background color
	 * @public
	 * @type string
	 * @default undefined
	 */
	this.color = {
	    R:0,
	    G:0,
	    B:0
	};
	//this.color = "#000000";
	
	/** Map background color trasparence
     * @public
     * @type string
     * @default undefined
     */
    this.trasparent = true;
    
    /** Map icon
     * @public
     * @type Icon
     * @default new Icon
     */
    this.icon = new Icon();
	
	/** Map html id
	 * @public 
	 * @type string
	 * @default undefined
	 */
	this.id = undefined;
	
	/** Map name
	 * @public 
	 * @type string
	 * @default undefined
	 */
	this.name = undefined;
	
	/** Map components objects array
     * @public
     * @type Array
     * @default new Array();
     */
    this.components = new Array();
    

    /** Add map component
     * 
     */
	this.addMapComponent = function(oComponentNode){
	    var oMapComponent = new MapComponent();
	    var oNode = new Node(oMapComponent);
	    var strMapComponentId = self.id +"_mapcomponent_"+ m_oMapComponentIdAssign.getId();
	    
	    oMapComponent.id = strMapComponentId;
	    oMapComponent.setReference(oComponentNode);
	    oMapComponent.setReferenceMap(self);
	    
	    //Push into map's components array
	    self.components.push(oNode);
	    
	    return oNode;
	};
	
	/** Remove map component
	 * 
	 */
	this.removeMapComponent = function(oNode){
        
        //Check if UIComponent contain current Map
        var bFind = false;
        for(var i=0; i<self.components.length; ++i){
            
            if(self.components[i] === oNode){
               
               //Check if is selected
               
               //Remove from array
               self.components.splice(i,1);
               
               bFind = true;
               break;
            }
        }
        if(!bFind){
            console.error("oNode is not contained in this Map");
            return 3;
        }
        
        //Nuovo ritorno!!!!
        return true;
    };
	
	/** JSON serialization
     *
     */
    this.encodeJSON = function(){
        //Set instance variables
        var oSerialized = {
            name: self.name,
            background: self.background,
            color: self.color,
            trasparent: self.trasparent,
            imageVisible: self.imageVisible,
            icon: self.icon,
            //Create new Array
            components: new Array()
        };

       // console.log(self.icon);
        
        //Set data parameters
        for(var i=0; i<self.components.length; ++i){
            var oMapComponent = self.components[i];
            oSerialized.components.push(oMapComponent.encodeJSON());
        }
        
        return oSerialized;
    };
	
	/** Overwrite toString() method
     * @returns {string} String that describe UIComponent object
     */
    this.toString = function(){
    	return "Map - ID: "+ self.id + "; name: "+ self.name +"; components length: "+ self.components.length;
    };

    //Initial operations
    init();
}