/** Preferences Manager (singleton)
 *
 * @class PreferencesManager
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 * 
 * @required <a href="http://jquery.com/">jQuery</a>
 * 
 */
"use strict";

var PreferencesManager = function() {
    
    /** Initialization function
     * 
     */

    this.init = function() {
        console.log("%cPreferences manager initialization","color:green");

        //Add view initialization event on document ready
        document.addEventListener("DOMContentLoaded",self._initView);

        //Load and fill protocols combo box from "/protocols/declare.json"
        $.ajax({
            dataType: "json",
            url: "../config.json",
            async: false,
            cache: false,
            success: self._onLoadConfig,
            error: function(jqXHR,textStatus,errorThrown){
                //Don't exist
                console.warn("Configuration file \"%c./config.json%c\" initialized","color:orange;","color:inherit");

                for(var k in m_oConfig){
                    self.setParameter(k,m_oConfig[k]);
                }
            }
        });
    };
    
    /** Instance
     * 
     */
    var self = this;

    /** App JSON data config (eg. language, recents)
     * @private
     */
    var m_oConfig = {
        language: "en",
        application:{
            name: "n.d.",
            version: "n.d."
        },
        view: "classic",
        recents: new Array(),
        currentProject: undefined,
        connectedUser: undefined,
        currentUser: undefined,
        //Preferences
        preferences:{
            "interface":{
                iconsScaleFactor: 1,
                snapEnabled: true,
                gridEnabled: true,
                grid: [20,20]
            }
        },
        checkExistingPrj: false,
        projectsDir: "",
        colorPresets: [ "#FF0000", "#00FF00", "#0000FF", "#FFFF00", "#00FFFF", "#00FFFF", "#00FFFF", "#00FFFF" ]
    };

    /**
     *
     */
    this._initView = function(){

        $.publish("/view/application/refresh/parameter",["view"]);
        $.publish("/view/application/refresh/parameter",["application"]);

        /*if(self.getParameter("view") != "maps" ){
            //Set view
            $.publish("/view/application/refresh/parameter",["view"]);
        }*/

    };

    /**
     *
     */
    this._onLoadConfig = function(oData){
        //console.warn("Configuration file \"%c./config.json%c\" initialized","color:orange;","color:inherit");
        //Exist
        $.extend(true,m_oConfig,oData);
    };

    /** Set parameter
     *
     */
    this.setParameter = function(key,val){
        console.log("Set \"./config.json\" file %c"+ key +"%c parameter","color:blue;","color:inherit;");

        m_oConfig[key] = val;

        //Check recent project
        if(key == "recents"){
            var aRecents = m_oConfig[key];

            //Check length (max 10)
            while(aRecents.length > 10){
                aRecents.pop();
            }
        }

        //Write config.json file
        if( !QT.writeFile("../config.json",JSON.stringify(m_oConfig)) ){
            console.error("Can't edit \"./config.json\" file");
        }

        //Publish changes for application view
        if(View.ready){
            $.publish("/view/application/refresh/parameter",[key]);
        }
    };

    /** Get config param
     * @param {string} key
     */
    this.getParameter = function(key){
        //console.log("Get \"./config.json\" file %c"+ key +"%c parameter","color:blue;","color:inherit;");
        return m_oConfig[key];
    };

    /** Add recent project
     *
     */
    this.addRecentProject = function(oProject){
        var aRecents = self.getParameter("recents");

        if(oProject.projectName == "temp_upload.eve" || oProject.projectName == "temp_download.eve"){
            return;
        }

        //Check if current project already exist
        for(var i=0; i<aRecents.length; ++i){
            if(aRecents[i].projectPath == oProject.projectPath){
                return;
            }
        }

        aRecents.unshift(oProject);

        //Check length (max 10)
        while(aRecents.length > 10){
            aRecents.pop();
        }

        self.setParameter("recents",aRecents);
    };

    /** Set interface preference
     *
     */
    this.setInterfacePreferences = function(sKey,sParam){
        var oPreferences = self.getParameter("preferences");

        oPreferences["interface"][sKey] = sParam;

        self.setParameter("preferences",oPreferences);
    };

    /** get interface preference
     *
     */
    this.getInterfacePreference = function(sKey){
        var oPreferences = self.getParameter("preferences");

        return oPreferences["interface"][sKey];
    };

    /** Set Application preference
     *
     */
    this.setApplicationPreference = function(sKey,sParam){
        var oApp = self.getParameter("application");

        oApp[sKey] = sParam;

        self.setParameter("application",oApp);
    };

    /** get Application preference
     *
     */
    this.getApplicationPreference = function(sKey){
        return self.getParameter("application")[sKey];
    };
    
    self.init();
};