/** Window class
 *
 * @class Window
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 */
"use strict";

function Window(windowType){
	
	var self = this;
	
	var init = function(){
	    console.log("%cWindow class initialization","color:green");
	    
	    //Create window
	    switch(windowType){
	        case "alert":
               m_oWindowType = new Alert(self);
               $(document).ready(function(e){
                   self.assignWindow($("#alert"));
               });
               break;
            case "confirm":
               m_oWindowType = new Confirm(self);
               $(document).ready(function(e){
                   self.assignWindow($("#confirm"));
               });
               break;
	        case "loader":
               m_oWindowType = new loader(self);
               $(document).ready(function(e){
                   self.assignWindow($("#loader"));
               });
               break;
	        case "iconEditor":
               m_oWindowType = new iconEditor(self);
               $(document).ready(function(e){
                   self.assignWindow($("#iconEditor"));
               });
               break;
            case "componentEditor":
               m_oWindowType = new componentEditor(self);
               $(document).ready(function(e){
                   self.assignWindow($("#componentEditor"));
               });
               break;
            case "etsProject":
               m_oWindowType = new etsProject(self);
               $(document).ready(function(e){
                   self.assignWindow($("#etsProject"));
               });
               break;
            case "scenarioEditor":
                m_oWindowType = new scenarioEditor(self);
                $(document).ready(function(e){
                    self.assignWindow($("#scenarioEditor"));
                });
                break;
            case "addComponentsList":
                m_oWindowType = new addComponentsList(self);
                $(document).ready(function(e){
                    self.assignWindow($("#addComponentsList"));
                });
                break;
            case "databaseManager":
               m_oWindowType = new databaseManager(self);
               $(document).ready(function(e){
                   self.assignWindow($("#databaseManager"));
               });
               break;
            case "userManager":
                m_oWindowType = new userManager(self);
                $(document).ready(function(e){
                    self.assignWindow($("#userManager"));
                });
                break;
            case "weekEditor":
                m_oWindowType = new weekEditor(self);
                $(document).ready(function(e){
                    self.assignWindow($("#weekEditor"));
                });
                break;
            case "yearEditor":
                m_oWindowType = new yearEditor(self);
                $(document).ready(function(e){
                    self.assignWindow($("#yearEditor"));
                });
                break;
            case "logicEditor":
                m_oWindowType = new logicEditor(self);
                $(document).ready(function(e){
                    self.assignWindow($("#logicEditor"));
                });
                break;
	        default:
	           console.warn("Window type: %c"+ windowType +"%c don't exist","color:blue;","color:inherit;");
	    }
	};
	
	/** Window's active node
	 * @type Node
	 */
	var m_oActiveNode = undefined;
	
	/** Extended window object (eg. iconEditor)
	 * 
     * @type {}
     */
    var m_oWindowType = undefined;
	
	/** Show scenario composition window
	 * @param oNode
	 * @param oNode oParam
	 *
	 */
	this.show = function(oNode,oParam){
	    //Set window active Node
	    self.setActiveNode(oNode);
	    
	    m_oWindowType.show(oParam);
	};
	
	/** Hide scenario composition window. Update memory area of active scenario component
	 */
	this.hide = function(){
	    
	    //Call inner window hide method
	    m_oWindowType.hide();
	    
	    //Set active node to undefined before close
	    if(m_oActiveNode != undefined){
	        m_oActiveNode = undefined;
	    }
	};
	
	/** Set Active Node
	 * @param oNode
	 * @returns {Node}
	 */
	this.setActiveNode = function(oNode){
	    if(!oNode instanceof Node){
	        console.error("Param type %c"+ typeof oNode +" can't be attached to window");
	        return undefined;
	    }
		m_oActiveNode = oNode;
		
		return m_oActiveNode;
	};
	
	/** Get Active Node
     * @param oNode
     * @returns {Node}
     */
    this.getActiveNode = function(){
        return m_oActiveNode;
    };

    /** Assign window HTML element 
     * @param $element jQuery element represent this window
     * @returns {Node}
     */
    this.assignWindow = function($element){
        m_oWindowType.assignWindow($element);
    };
    
	/** Initialization */
	init();
}